import unittest
from datetime import datetime
from zoneinfo import ZoneInfo
import aemo_to_tariff.sapower as sapower

class TestSAPower(unittest.TestCase):
    def test_some_sapower_functionality(self):
        interval_time = datetime(2025, 2, 20, 9, 10, tzinfo=ZoneInfo('Australia/Adelaide'))
        tariff_code = 'RTOU'
        rrp = -100.0
        expected_price = 10.26535477
        price = sapower.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 0.96, expected_price, places=1, msg=f"Price: {price}, Expected: {expected_price}, Loss Factor: {loss_factor}")

    def test_later_day(self):
        interval_time = datetime(2025, 2, 20, 15, 10, tzinfo=ZoneInfo('Australia/Adelaide'))
        tariff_code = 'RTOU'
        rrp = -76.53
        expected_price = 13.03587882
        price = sapower.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1, expected_price, places=1, msg=f"Price: {price}, Expected: {expected_price}, Loss Factor: {loss_factor}")
    
    def test_two_way_tou_peak(self):
        interval_time = datetime(2025, 2, 20, 18, 10, tzinfo=ZoneInfo('Australia/Adelaide'))
        tariff_code = 'RELE2W'
        rrp = -76.53
        expected_price = 29.705328600000005
        price = sapower.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.1678, expected_price, places=1, msg=f"Price: {price}, Expected: {expected_price}, Loss Factor: {loss_factor}")
    
    def test_two_way_tou_feed_peak(self):
        interval_time = datetime(2025, 2, 20, 18, 10, tzinfo=ZoneInfo('Australia/Adelaide'))
        tariff_code = 'RELE2W'
        rrp = 100
        expected_price = 22.36
        price = sapower.convert_feed_in_tariff(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(price, expected_price, places=1)
    
    def test_two_way_tou_feed_off_peak(self):
        interval_time = datetime(2025, 2, 20, 13, 10, tzinfo=ZoneInfo('Australia/Adelaide'))
        tariff_code = 'RELE2W'
        rrp = 100
        expected_price = 9.0
        price = sapower.convert_feed_in_tariff(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price, expected_price, places=1, msg=f"Price: {price}, Expected: {expected_price}, Loss Factor: {loss_factor}")

    def test_night_tou_tariff(self):
        interval_time = datetime(2025, 3, 30, 2, 55, tzinfo=ZoneInfo('Australia/Adelaide'))
        tariff_code = 'RTOU'
        rrp = 9.4
        expected_price = 11.2 - 1.7156
        price = sapower.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.02, expected_price, places=1, msg=f"Price: {price}, Expected: {expected_price}, Loss Factor: {loss_factor}")

    def test_morning_zero_twoway_tariff(self):
        interval_time = datetime(2025, 5, 7, 8, 20, tzinfo=ZoneInfo('Australia/Adelaide'))
        tariff_code = 'RELE2W'
        dlf = 1.1678
        rrp = 0.0
        expected_price = 34.7445
        price = sapower.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.05, expected_price, places=1, msg=f"Price: {price}, Expected: {expected_price}, Loss Factor: {loss_factor}")
        feed_in_price = sapower.convert_feed_in_tariff(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(feed_in_price, 0, places=2, msg=f"Feed-in Price: {feed_in_price}, Expected: 1.0")

    def test_feb_twoway_tariff(self):
        interval_time = datetime(2025, 2, 7, 18, 20, tzinfo=ZoneInfo('Australia/Adelaide'))
        tariff_code = 'RELE2W'
        dlf = 1.1678
        rrp = 0.0
        feed_in_price = sapower.convert_feed_in_tariff(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(feed_in_price, 12.36, places=2, msg=f"Feed-in Price: {feed_in_price}, Expected: 1.0")

    def test_feb_twoway_utc_tariff(self):
        interval_time = datetime(2025, 2, 7, 8, 20, tzinfo=ZoneInfo('UTC'))
        tariff_code = 'RELE2W'
        dlf = 1.1678
        rrp = 0.0
        feed_in_price = sapower.convert_feed_in_tariff(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(feed_in_price, 12.36, places=2, msg=f"Feed-in Price: {feed_in_price}, Expected: 1.0")

    def test_morning_tou_tariff(self):
        interval_time = datetime(2025, 3, 30, 8, 55, tzinfo=ZoneInfo('Australia/Adelaide'))
        tariff_code = 'RTOU'
        rrp = 19.28
        expected_price = 25.45 - 1.7156
        price = sapower.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.05, expected_price, places=1, msg=f"Price: {price}, Expected: {expected_price}, Loss Factor: {loss_factor}")


import unittest
from zoneinfo import ZoneInfo
from datetime import datetime
from aemo_to_tariff.victoria import time_zone, convert, get_daily_fee

class TestVictoria(unittest.TestCase):
    def test_convert(self):
        interval_time = datetime(2023, 7, 15, 10, 0, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'VICR_TOU'
        rrp = 100.0
        expected_price = 40.0
        price = convert(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(price, expected_price, places=2)

    def test_get_daily_fee(self):
        tariff_code = 'VICR_TOU'
        annual_usage = 20000
        expected_fee = 1.2
        fee = get_daily_fee(tariff_code, annual_usage)
        self.assertEqual(fee, expected_fee)



import unittest
from datetime import datetime
from zoneinfo import ZoneInfo
import aemo_to_tariff.essential as essential

class TestEssentualPower(unittest.TestCase):
    def test_some_essential_functionality(self):
        interval_time = datetime(2025, 2, 20, 9, 10, tzinfo=ZoneInfo('Australia/Sydney'))
        tariff_code = 'BLNT3AL'
        rrp = -100.0
        expected_price = 3.858878319999999
        price = essential.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.168, expected_price, places=1)

    def test_seven_pm(self):
        # 2025-05-12 19:25:00+10
        interval_time = datetime(2025, 5, 12, 19, 25, tzinfo=ZoneInfo('Australia/Sydney'))
        tariff_code = 'BLND1AR'
        rrp = 135.36
        expected_price = 37.60955297
        price = essential.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.68, expected_price, places=1)

    def test_seven_am(self):
        # 2025-05-12 07:25:00+10
        interval_time = datetime(2025, 5, 12, 7, 25, tzinfo=ZoneInfo('Australia/Sydney'))
        tariff_code = 'BLND1AR'
        rrp = 153.88
        expected_price = 34.06917696
        price = essential.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.6, expected_price, places=1)

    def test_four_am(self):
        # 2025-05-10 04:15:00+10
        interval_time = datetime(2025, 5, 20, 4, 15, tzinfo=ZoneInfo('Australia/Sydney'))
        tariff_code = 'BLND1AR'
        rrp = 67.73
        expected_price = 15.23709699
        price = essential.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.48, expected_price, places=1)

    def test_later_day(self):
        # 2025-05-10 12:25:00+10
        interval_time = datetime(2025, 5, 20, 12, 25, tzinfo=ZoneInfo('Australia/Sydney'))
        tariff_code = 'BLND1AR'
        rrp = -16.13
        expected_price = 6.21313241
        price = essential.convert(interval_time, tariff_code, rrp)
        loss_factor = expected_price / price
        self.assertAlmostEqual(price * 1.447, expected_price, places=1)



# test/test_ausnet.py

import unittest
from datetime import datetime
from zoneinfo import ZoneInfo
from aemo_to_tariff.ausnet import convert_feed_in_tariff, time_zone, convert, calculate_demand_fee

class TestAusNet(unittest.TestCase):
    def test_nast11s_peak(self):
        interval_time = datetime(2025, 6, 12, 10, 0, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'NAST11S'
        rrp = 100.0  # $/MWh → 10 c/kWh
        expected_price = 10.0 + 14.6  # rrp + peak rate
        actual_price = convert(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(actual_price, expected_price, places=0)

    def test_nast11s_offpeak(self):
        interval_time = datetime(2025, 6, 12, 3, 0, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'NAST11S'
        # amber_sell_price': 47.33081,
        # amber_buy_price': 63.83136,
        # amber_spot_per_kwh': 48.07081,
        rrp = 480.0  # $/MWh → 8 c/kWh
        expected_price = 63.0
        actual_price = convert(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(actual_price, expected_price, places=0)

    def test_nast11s_feed_in_passthrough(self):
        interval_time = datetime(2025, 6, 12, 14, 0, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'NAST11S'
        rrp = 120.0
        expected = 12.0  # passthrough
        actual = convert_feed_in_tariff(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(actual, expected, places=2)

    def test_nast11s_demand_fee_default(self):
        tariff_code = 'NAST11S'
        demand_kw = 5
        fee = calculate_demand_fee(tariff_code, demand_kw, days=30)
        self.assertEqual(fee, 0.0)  # no demand charge defined

    def test_pre_peak_june(self):
        interval_time = datetime(2025, 6, 18, 16, 0, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'NAST11S'
        sell_price = 21.16197
        expected = 48.84239
        rrp = 209
        actual = convert(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(actual, expected, places=0)
        actual = convert_feed_in_tariff(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(actual, sell_price, places=0)

    def test_noon_peak_june(self):
        interval_time = datetime(2025, 6, 12, 11, 0, tzinfo=ZoneInfo(time_zone()))
        tariff_code = 'NAST11S'
        expected = 28.43681
        rrp = 139.92
        actual = convert(interval_time, tariff_code, rrp)
        self.assertAlmostEqual(actual, expected, places=0)